//***************************************************************************
// File
// Date tt.mm.jj - #1
// 
//--------------------------------------------------------------------------
//
//***************************************************************************

//***************************************************************************
// HOETO build
//***************************************************************************

// g++ -ggdb -I. com.cc common.cc -o comtest

//***************************************************************************
// Includes
//***************************************************************************

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <time.h>
#include <termios.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "common.hpp"

//***************************************************************************
// Commands
//***************************************************************************

const char* cmdGetTime = "GETTIME;";
const char* cmdGhostCar = "RGC";
int fd = 0;

//***************************************************************************
// Dispatch
//***************************************************************************

int dispatch(char* buf, int& value)
{
   // Examples:
   //     "BOARDTIME=750205;"
   //     "DI:754041=2999;"

   char* p = buf;
   char* cmd = buf;
   char* val = 0;
   char* t = 0;

   if ((p = strchr(p, ':')))
   {
      *p = 0;
      p++;
      t = p;
   }
   else
      p = buf;

   if ((p = strchr(p, '=')))
   {
      *p = 0;
      p++;
      val = p;
   }
   else
      p = buf;

   if (!(p = strchr(p, ';')))
      return -1;

   *p = 0;

   if (val)
   {
      if (strncasecmp(val, "0x", 2)  == 0)
         value = strtoll(val+2, 0, 16);
      else
         value = atoll(val);
   }

   return 0;
}

//***************************************************************************
// To Bin Str
//***************************************************************************

const char* toBinStr(unsigned int value, char* mask)
{
   const int bytes = 4;
   const int bits = bytes * 8;
   int bit = 1;

   for (int i = 0, b = 0; b < bits; i++, b++)
   {
      if (!(b % 8) && b)
      {
         mask[bits+bytes-i-2] = ':';
         i++;
      }
      
      mask[bits+bytes-i-2] = (value & bit) ? '1' : '0';     
      bit = bit << 1;
   }

   mask[bits+bytes-1] = 0;

   return mask;
}

//***************************************************************************
// Read Line
//***************************************************************************

char* read(char* line, char lookahed = 0)
{
   int count = 0;
   char b;
   time_t start = time(0);

   *line = 0;

   if (lookahed && lookahed != '\n')
   {
      line[0] = lookahed;
      line[1] = 0;
      count++;
   }
   else
      return "";

   while (true)
   {
      if (start+1 < time(0))
      {
         tell(eloAlways, "aborting read!");
         break;
      }

      if (::read(fd, &b, 1) <= 0)
      {
         usleep(10000);
         continue;
      }

      if (b == '\r')
         break;

      line[count] = b;
      count++;      
   }

   line[count] = 0;

   tell(eloDebug, "Got: '%s'", line);

   return line;
}

//***************************************************************************
// Main
//***************************************************************************

int main(int argc, char** argv)
{
   struct termios oldtio, newtio;
   int isRunning = true;
   int res;
   char b;
   int v = 1;
   int pin = 0;
   char line[1000];
   char tmp[1000];
   char buf[100];
   int value = 0;

   char* device = "/dev/ttyUSB0";
   // char* device = "/dev/ttyS0";

   if ((fd = open(device, O_RDWR | O_NOCTTY)) < 0)
   {
      perror(device); 
      return -1;
   }

   theEloquence = 1;

   // configure serial line

   tcgetattr(fd, &oldtio);
   bzero(&newtio, sizeof(newtio));
   newtio.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
   newtio.c_iflag = IGNPAR | PARMRK;
   newtio.c_oflag = 0;
   newtio.c_lflag = 0;
   tcflush(fd, TCIFLUSH);
   tcsetattr(fd, TCSANOW, &newtio);

   // first ask abut the boardtime

   write(fd, cmdGetTime, strlen(cmdGetTime));
   tell(eloAlways, "-> [%s]", cmdGetTime);

   if (argc >= 2 && argv[1][0] == 'r')
   {
      sprintf(tmp, "%s:5=200;", cmdGhostCar);
      write(fd, tmp, strlen(tmp));
      tell(eloAlways, "-> [%s]", tmp);
   }

   if (argc >= 2 && argv[1][0] == 's')
   {
      char c[100];
      
      tell(eloAlways, "send mode");

      while (isRunning)
      {
         fgets(c, 80, stdin);

         sprintf(tmp, "AO%d=%d;", 5, atoi(c));
         write(fd, tmp, strlen(tmp));
         tell(eloAlways, "-> [%s]", tmp);
      }  

      tell(eloAlways, "END");

      isRunning = false;
      goto EXIT;
   }

   while (isRunning)
   { 
      if (::read(fd, &b, 1) <= 0)
      {
         usleep(10000);
         tell(eloDebug, "looping");

         continue;
      }

      read(line, b);

      if (*line == 0)
         continue;

      strcpy(tmp, line);

      if (dispatch(tmp, value) == 0)
      {
         if (!strstr(line, "DEBUG"))
         {
            toBinStr(value, buf);
            tell(eloAlways, "<- [%-30s] (%s)", line, buf);
         }
         else
         {
            if (strstr(line, "DEBUG:"))
            {
               toBinStr(value, buf);
               tell(eloAlways, "<- [%-30s] (%s) (0x%X)", line, buf, value);
            }
            else
               tell(eloAlways, "<- [%s]", line);
         }

         continue;

         pin++;

         if (pin == 16)
         {
            v = v == 1 ? 0 : 1;
            pin = 0;
         }

         sprintf(tmp, "DO%d=%d;", pin, v);
         write(fd, tmp, strlen(tmp));
         tell(eloAlways, "-> [%s]", tmp);
      }
   }

 EXIT:

   // restore the old settings and close

   tcsetattr(fd, TCSANOW, &oldtio);
   ::close(fd);

   return 0;
}
